@@ -2,7 +2,7 @@
  * netbios.c
  *
  * Copyright (C) 2009-2011 by ipoque GmbH
- * Copyright (C) 2011-15 - ntop.org
+ * Copyright (C) 2011-17 - ntop.org
  *
  * This file is part of nDPI, an open source deep packet inspection
  * library based on the OpenDPI and PACE technology by ipoque GmbH
@@ -27,8 +27,12 @@
 
 #ifdef NDPI_PROTOCOL_NETBIOS
 
+struct netbios_header {
+  u_int16_t transaction_id, flags, questions, answer_rrs, authority_rrs, additional_rrs;
+};
+
 /* The function below has been inherited by tcpdump */
-static int netbios_name_interpret(char *in, char *out, u_int out_len) {
+int ndpi_netbios_name_interpret(char *in, char *out, u_int out_len) {
   int ret = 0, len;
   char *b;
   
@@ -46,8 +50,11 @@ static int netbios_name_interpret(char *in, char *out, u_int out_len) {
     }
 
     *out = ((in[0]-'A')<<4) + (in[1]-'A');
+        
     in += 2;
-    out++, ret++;
+
+    if(isprint(*out))
+      out++, ret++;
   }
 
   *out = 0;
@@ -63,6 +70,7 @@ static int netbios_name_interpret(char *in, char *out, u_int out_len) {
 static void ndpi_int_netbios_add_connection(struct ndpi_detection_module_struct
 					    *ndpi_struct, struct ndpi_flow_struct *flow)
 {
+  
   ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_NETBIOS, NDPI_PROTOCOL_UNKNOWN);
 }
 
@@ -70,28 +78,30 @@ static void ndpi_int_netbios_add_connection(struct ndpi_detection_module_struct
 void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
 {
   struct ndpi_packet_struct *packet = &flow->packet;
-	
-  //      struct ndpi_id_struct         *src=ndpi_struct->src;
-  //      struct ndpi_id_struct         *dst=ndpi_struct->dst;
-
-
   u_int16_t dport;
-
-  if (packet->udp != NULL) {
+  char name[64];
+  
+  if(packet->udp != NULL) {
     dport = ntohs(packet->udp->dest);
 
     NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG, "netbios udp start\n");
 
     /*check standard NETBIOS over udp to port 137  */
-    if ((dport == 137 || 0) && packet->payload_packet_len >= 50) {
+    if((dport == 137 || 0) && packet->payload_packet_len >= 50) {
+      struct netbios_header h;
 
+      memcpy(&h, packet->payload, sizeof(struct netbios_header));
+      h.transaction_id = ntohs(h.transaction_id), h.flags = ntohs(h.flags),
+	h.questions = ntohs(h.questions), h.answer_rrs = ntohs(h.answer_rrs),
+	h.authority_rrs = ntohs(h.authority_rrs), h.additional_rrs = ntohs(h.additional_rrs);
+      
       NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 	       NDPI_LOG_DEBUG, "found netbios port 137 and payload_packet_len 50\n");
 
-      if (ntohs(get_u_int16_t(packet->payload, 2)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 4)) == 1 &&
-	  ntohs(get_u_int16_t(packet->payload, 6)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 8)) == 0 && ntohs(get_u_int16_t(packet->payload, 10)) == 0) {
+      if(h.flags == 0 &&
+	  h.questions == 1 &&
+	  h.answer_rrs == 0 &&
+	  h.authority_rrs == 0 && h.additional_rrs == 0) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG, "found netbios with questions = 1 and answers = 0, authority = 0  \n");
@@ -99,10 +109,24 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	ndpi_int_netbios_add_connection(ndpi_struct, flow);
 	return;
       }
-      if (packet->payload[2] == 0x80 &&
-	  ntohs(get_u_int16_t(packet->payload, 4)) == 1 &&
-	  ntohs(get_u_int16_t(packet->payload, 6)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 8)) == 0 && ntohs(get_u_int16_t(packet->payload, 10)) == 1) {
+      if(((h.flags & 0x8710) == 0x10) &&
+	  h.questions == 1 &&
+	  h.answer_rrs == 0 &&
+	  h.authority_rrs == 0) {
+
+	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
+		 NDPI_LOG_DEBUG, "found netbios with questions = 1 and answers = 0, authority = 0 and broadcast \n");
+	
+	if(ndpi_netbios_name_interpret((char*)&packet->payload[12], name, sizeof(name)) > 0)
+	  snprintf((char*)flow->host_server_name, sizeof(flow->host_server_name)-1, "%s", name);
+
+	ndpi_int_netbios_add_connection(ndpi_struct, flow);
+	return;
+      }
+      if(packet->payload[2] == 0x80 &&
+	  h.questions == 1 &&
+	  h.answer_rrs == 0 &&
+	  h.authority_rrs == 0 && h.additional_rrs == 1) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG, "found netbios with questions = 1 and answers, authority, additional = 0  \n");
@@ -110,10 +134,10 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	ndpi_int_netbios_add_connection(ndpi_struct, flow);
 	return;
       }
-      if (ntohs(get_u_int16_t(packet->payload, 2)) == 0x4000 &&
-	  ntohs(get_u_int16_t(packet->payload, 4)) == 1 &&
-	  ntohs(get_u_int16_t(packet->payload, 6)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 8)) == 0 && ntohs(get_u_int16_t(packet->payload, 10)) == 1) {
+      if(h.flags == 0x4000 &&
+	  h.questions == 1 &&
+	  h.answer_rrs == 0 &&
+	  h.authority_rrs == 0 && h.additional_rrs == 1) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG, "found netbios with questions = 1 and answers = 0, authority = 0  \n");
@@ -121,10 +145,10 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	ndpi_int_netbios_add_connection(ndpi_struct, flow);
 	return;
       }
-      if (ntohs(get_u_int16_t(packet->payload, 2)) == 0x8400 &&
-	  ntohs(get_u_int16_t(packet->payload, 4)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 6)) == 1 &&
-	  ntohs(get_u_int16_t(packet->payload, 8)) == 0 && ntohs(get_u_int16_t(packet->payload, 10)) == 0) {
+      if(h.flags == 0x8400 &&
+	  h.questions == 0 &&
+	  h.answer_rrs == 1 &&
+	  h.authority_rrs == 0 && h.additional_rrs == 0) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG,
@@ -133,10 +157,10 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	ndpi_int_netbios_add_connection(ndpi_struct, flow);
 	return;
       }
-      if (ntohs(get_u_int16_t(packet->payload, 2)) == 0x8500 &&
-	  ntohs(get_u_int16_t(packet->payload, 4)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 6)) == 1 &&
-	  ntohs(get_u_int16_t(packet->payload, 8)) == 0 && ntohs(get_u_int16_t(packet->payload, 10)) == 0) {
+      if(h.flags == 0x8500 &&
+	  h.questions == 0 &&
+	  h.answer_rrs == 1 &&
+	  h.authority_rrs == 0 && h.additional_rrs == 0) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG,
@@ -145,10 +169,10 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	ndpi_int_netbios_add_connection(ndpi_struct, flow);
 	return;
       }
-      if (ntohs(get_u_int16_t(packet->payload, 2)) == 0x2910 &&
-	  ntohs(get_u_int16_t(packet->payload, 4)) == 1 &&
-	  ntohs(get_u_int16_t(packet->payload, 6)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 8)) == 0 && ntohs(get_u_int16_t(packet->payload, 10)) == 1) {
+      if(h.flags == 0x2910 &&
+	  h.questions == 1 &&
+	  h.answer_rrs == 0 &&
+	  h.authority_rrs == 0 && h.additional_rrs == 1) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG,
@@ -157,10 +181,10 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	ndpi_int_netbios_add_connection(ndpi_struct, flow);
 	return;
       }
-      if (ntohs(get_u_int16_t(packet->payload, 2)) == 0xAD86 &&
-	  ntohs(get_u_int16_t(packet->payload, 4)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 6)) == 1 &&
-	  ntohs(get_u_int16_t(packet->payload, 8)) == 0 && ntohs(get_u_int16_t(packet->payload, 10)) == 0) {
+      if(h.flags == 0xAD86 &&
+	  h.questions == 0 &&
+	  h.answer_rrs == 1 &&
+	  h.authority_rrs == 0 && h.additional_rrs == 0) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG,
@@ -169,10 +193,10 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	ndpi_int_netbios_add_connection(ndpi_struct, flow);
 	return;
       }
-      if (ntohs(get_u_int16_t(packet->payload, 2)) == 0x0110 &&
-	  ntohs(get_u_int16_t(packet->payload, 4)) == 1 &&
-	  ntohs(get_u_int16_t(packet->payload, 6)) == 0 &&
-	  ntohs(get_u_int16_t(packet->payload, 8)) == 0 && ntohs(get_u_int16_t(packet->payload, 10)) == 0) {
+      if(h.flags == 0x0110 &&
+	  h.questions == 1 &&
+	  h.answer_rrs == 0 &&
+	  h.authority_rrs == 0 && h.additional_rrs == 0) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG,
@@ -182,11 +206,11 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	return;
       }
 
-      if ((ntohs(get_u_int16_t(packet->payload, 2)) & 0xf800) == 0) {
+      if((h.flags & 0xf800) == 0) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG, "possible netbios name query request\n");
 
-	if (get_u_int16_t(packet->payload, 4) == htons(1) &&
+	if(get_u_int16_t(packet->payload, 4) == htons(1) &&
 	    get_u_int16_t(packet->payload, 6) == 0 &&
 	    get_u_int16_t(packet->payload, 8) == 0 && get_u_int16_t(packet->payload, 10) == 0) {
 
@@ -196,13 +220,13 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	  NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		   "possible netbios name query request, one question\n");
 
-	  if (packet->payload_packet_len == 12 + 1 + name_length + 1 + 2 + 2) {
+	  if(packet->payload_packet_len == 12 + 1 + name_length + 1 + 2 + 2) {
 
 	    NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		     "possible netbios name query request, length matches\n");
 
 	    /* null terminated? */
-	    if (packet->payload[12 + name_length + 1] == 0 &&
+	    if(packet->payload[12 + name_length + 1] == 0 &&
 		get_u_int16_t(packet->payload, 12 + name_length + 2) == htons(0x0020) &&
 		get_u_int16_t(packet->payload, 12 + name_length + 4) == htons(0x0001)) {
 
@@ -213,12 +237,11 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	    }
 	  }
 	}
-      } else if ((ntohs(get_u_int16_t(packet->payload, 2)) & 0xf800) == 0x8000) {
-
+      } else if((h.flags & 0xf800) == 0x8000) {
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		 "possible netbios name query response\n");
 
-	if (get_u_int16_t(packet->payload, 4) == 0 &&
+	if(get_u_int16_t(packet->payload, 4) == 0 &&
 	    get_u_int16_t(packet->payload, 6) == htons(1) &&
 	    get_u_int16_t(packet->payload, 8) == 0 && get_u_int16_t(packet->payload, 10) == 0) {
 
@@ -228,13 +251,13 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	  NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		   "possible netbios positive name query response, one answer\n");
 
-	  if (packet->payload_packet_len >= 12 + 1 + name_length + 1 + 2 + 2) {
+	  if(packet->payload_packet_len >= 12 + 1 + name_length + 1 + 2 + 2) {
 
 	    NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		     "possible netbios name query response, length matches\n");
 
 	    /* null terminated? */
-	    if (packet->payload[12 + name_length + 1] == 0 &&
+	    if(packet->payload[12 + name_length + 1] == 0 &&
 		get_u_int16_t(packet->payload, 12 + name_length + 2) == htons(0x0020) &&
 		get_u_int16_t(packet->payload, 12 + name_length + 4) == htons(0x0001)) {
 
@@ -244,7 +267,7 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	      return;
 	    }
 	  }
-	} else if (get_u_int16_t(packet->payload, 4) == 0 &&
+	} else if(get_u_int16_t(packet->payload, 4) == 0 &&
 		   get_u_int16_t(packet->payload, 6) == 0 &&
 		   get_u_int16_t(packet->payload, 8) == 0 && get_u_int16_t(packet->payload, 10) == 0) {
 
@@ -254,13 +277,13 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	  NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		   "possible netbios negative name query response, one answer\n");
 
-	  if (packet->payload_packet_len >= 12 + 1 + name_length + 1 + 2 + 2) {
+	  if(packet->payload_packet_len >= 12 + 1 + name_length + 1 + 2 + 2) {
 
 	    NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		     "possible netbios name query response, length matches\n");
 
 	    /* null terminated? */
-	    if (packet->payload[12 + name_length + 1] == 0 &&
+	    if(packet->payload[12 + name_length + 1] == 0 &&
 		get_u_int16_t(packet->payload, 12 + name_length + 2) == htons(0x000A) &&
 		get_u_int16_t(packet->payload, 12 + name_length + 4) == htons(0x0001)) {
 
@@ -270,7 +293,7 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	      return;
 	    }
 	  }
-	} else if (get_u_int16_t(packet->payload, 4) == 0 &&
+	} else if(get_u_int16_t(packet->payload, 4) == 0 &&
 		   get_u_int16_t(packet->payload, 6) == 0 &&
 		   get_u_int16_t(packet->payload, 8) == htons(1) && get_u_int16_t(packet->payload, 10) == htons(1)) {
 
@@ -280,13 +303,13 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 	  NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		   "possible netbios redirect name query response, one answer\n");
 
-	  if (packet->payload_packet_len >= 12 + 1 + name_length + 1 + 2 + 2) {
+	  if(packet->payload_packet_len >= 12 + 1 + name_length + 1 + 2 + 2) {
 
 	    NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG,
 		     "possible netbios name query response, length matches\n");
 
 	    /* null terminated? */
-	    if (packet->payload[12 + name_length + 1] == 0 &&
+	    if(packet->payload[12 + name_length + 1] == 0 &&
 		get_u_int16_t(packet->payload, 12 + name_length + 2) == htons(0x0002) &&
 		get_u_int16_t(packet->payload, 12 + name_length + 4) == htons(0x0001)) {
 
@@ -305,25 +328,23 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 
     /*netbios header token from http://www.protocolbase.net/protocols/protocol_NBDGM.php */
 
-    if ((dport == 138) &&
+    if((dport == 138) &&
 	packet->payload_packet_len >= 14 &&
 	ntohs(get_u_int16_t(packet->payload, 10)) == packet->payload_packet_len - 14) {
 
       NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 	       NDPI_LOG_DEBUG, "found netbios port 138 and payload length >= 112 \n");
 
-      if (packet->payload[0] >= 0x11 && packet->payload[0] <= 0x16) {
+      if(packet->payload[0] >= 0x11 && packet->payload[0] <= 0x16) {
 
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG, "found netbios with MSG-type 0x11,0x12,0x13,0x14,0x15 or 0x16\n");
 
-	if (ntohl(get_u_int32_t(packet->payload, 4)) == ntohl(packet->iph->saddr)) {
-	  char name[64];
-
+	if(ntohl(get_u_int32_t(packet->payload, 4)) == ntohl(packet->iph->saddr)) {
 	  NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		   NDPI_LOG_DEBUG, "found netbios with checked ip-address.\n");
 
-	  if(netbios_name_interpret((char*)&packet->payload[12], name, sizeof(name)) > 0)
+	  if(ndpi_netbios_name_interpret((char*)&packet->payload[12], name, sizeof(name)) > 0)
 	    snprintf((char*)flow->host_server_name, sizeof(flow->host_server_name)-1, "%s", name);
 
 	  ndpi_int_netbios_add_connection(ndpi_struct, flow);
@@ -333,22 +354,22 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
     }
   }
 
-  if (packet->tcp != NULL) {
+  if(packet->tcp != NULL) {
     dport = ntohs(packet->tcp->dest);
 
     NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG, "netbios tcp start\n");
 
     /* destination port must be 139 */
-    if (dport == 139) {
+    if(dport == 139) {
 
       NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG, "found netbios with destination port 139\n");
 
       /* payload_packet_len must be 72 */
-      if (packet->payload_packet_len == 72) {
+      if(packet->payload_packet_len == 72) {
 	NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		 NDPI_LOG_DEBUG, "found netbios with payload_packen_len = 72. \n");
 
-	if (packet->payload[0] == 0x81 && packet->payload[1] == 0 && ntohs(get_u_int16_t(packet->payload, 2)) == 68) {
+	if(packet->payload[0] == 0x81 && packet->payload[1] == 0 && ntohs(get_u_int16_t(packet->payload, 2)) == 68) {
 	  NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct,
 		   NDPI_LOG_DEBUG,
 		   "found netbios with session request = 81, flags=0 and length od following bytes = 68. \n");
@@ -363,10 +384,8 @@ void ndpi_search_netbios(struct ndpi_detection_module_struct *ndpi_struct, struc
 
   NDPI_LOG(NDPI_PROTOCOL_NETBIOS, ndpi_struct, NDPI_LOG_DEBUG, "exclude netbios\n");
   NDPI_ADD_PROTOCOL_TO_BITMASK(flow->excluded_protocol_bitmask, NDPI_PROTOCOL_NETBIOS);
-
 }
 
-
 void init_netbios_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask)
 {
   ndpi_set_bitmask_protocol_detection("NETBIOS", ndpi_struct, detection_bitmask, *id,
