@@ -1,7 +1,7 @@
 /*
  * skype.c
  *
- * Copyright (C) 2011-15 - ntop.org
+ * Copyright (C) 2017 - ntop.org
  *
  * nDPI is free software: you can redistribute it and/or modify
  * it under the terms of the GNU Lesser General Public License as published by
@@ -17,56 +17,21 @@
  * along with nDPI.  If not, see <http://www.gnu.org/licenses/>.
  *
  */
-
-
 #include "ndpi_api.h"
 
 #ifdef NDPI_PROTOCOL_SKYPE
 
-static u_int8_t is_skype_host(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t host) {
-  struct in_addr pin;
-  
-  pin.s_addr = host;
-  
-  return((ndpi_network_ptree_match(ndpi_struct, &pin) == NDPI_PROTOCOL_SKYPE) ? 1 : 0);
-}
-
-u_int8_t is_skype_flow(struct ndpi_detection_module_struct *ndpi_struct,
-		       struct ndpi_flow_struct *flow) {
-  struct ndpi_packet_struct *packet = &flow->packet;
-	
-  if(packet->iph) {
-    /*
-      Skype connections are identified by some SSL-like communications
-      without SSL certificate being exchanged
-    */	
-    if(is_skype_host(ndpi_struct, packet->iph->saddr)
-       || is_skype_host(ndpi_struct, packet->iph->daddr)) {
-      return(1);
-    }
-  }
-
-  return(0);
-}
-
 static void ndpi_check_skype(struct ndpi_detection_module_struct *ndpi_struct, struct ndpi_flow_struct *flow)
 {
   struct ndpi_packet_struct *packet = &flow->packet;
   // const u_int8_t *packet_payload = packet->payload;
   u_int32_t payload_len = packet->payload_packet_len;
 
+
   if(flow->host_server_name[0] != '\0')
     return;
 
-  /*
-    Skype AS8220
-    212.161.8.0/24
-  */
-  if(is_skype_flow(ndpi_struct, flow)) {
-    ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SKYPE, NDPI_PROTOCOL_UNKNOWN);
-    return;
-  }
-
+  // UDP check
   if(packet->udp != NULL) {
     flow->l4.udp.skype_packet_id++;
 
@@ -75,20 +40,20 @@ static void ndpi_check_skype(struct ndpi_detection_module_struct *ndpi_struct, s
 
       /* skype-to-skype */
       if(dport != 1119) /* It can be confused with battle.net */ {
-	if(((payload_len == 3) && ((packet->payload[2] & 0x0F)== 0x0d))
-	   || ((payload_len >= 16)
-	       && (packet->payload[0] != 0x30) /* Avoid invalid SNMP detection */
-	       && (packet->payload[2] == 0x02))) {
+	if(((payload_len == 3) && ((packet->payload[2] & 0x0F)== 0x0d)) ||
+	   ((payload_len >= 16)
+	    && (packet->payload[0] != 0x30) /* Avoid invalid SNMP detection */
+	    && (packet->payload[2] == 0x02))) {
 	  NDPI_LOG(NDPI_PROTOCOL_SKYPE, ndpi_struct, NDPI_LOG_DEBUG, "Found skype.\n");
 	  ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SKYPE, NDPI_PROTOCOL_UNKNOWN);
 	}
       }
-
       return;
     }
-
     NDPI_ADD_PROTOCOL_TO_BITMASK(flow->excluded_protocol_bitmask, NDPI_PROTOCOL_SKYPE);
     return;
+
+    // TCP check
   } else if(packet->tcp != NULL) {
     flow->l4.tcp.skype_packet_id++;
 
@@ -99,11 +64,15 @@ static void ndpi_check_skype(struct ndpi_detection_module_struct *ndpi_struct, s
 	      && flow->l4.tcp.seen_syn
 	      && flow->l4.tcp.seen_syn_ack
 	      && flow->l4.tcp.seen_ack) {
-      if((payload_len == 8) || (payload_len == 3)) {
-	//printf("[SKYPE] %u/%u\n", ntohs(packet->tcp->source), ntohs(packet->tcp->dest));
+
+      if((payload_len == 8) || (payload_len == 3) || (payload_len == 17)) {
+	// printf("[SKYPE] payload_len=%u\n", payload_len);
+	/* printf("[SKYPE] %u/%u\n", ntohs(packet->tcp->source), ntohs(packet->tcp->dest)); */
 
 	NDPI_LOG(NDPI_PROTOCOL_SKYPE, ndpi_struct, NDPI_LOG_DEBUG, "Found skype.\n");
 	ndpi_set_detected_protocol(ndpi_struct, flow, NDPI_PROTOCOL_SKYPE, NDPI_PROTOCOL_UNKNOWN);
+      } else {
+	// printf("NO [SKYPE] payload_len=%u\n", payload_len);
       }
 
       /* printf("[SKYPE] [id: %u][len: %d]\n", flow->l4.tcp.skype_packet_id, payload_len);  */
@@ -126,7 +95,7 @@ void ndpi_search_skype(struct ndpi_detection_module_struct *ndpi_struct, struct
 }
 
 
-void init_skype_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask) 
+void init_skype_dissector(struct ndpi_detection_module_struct *ndpi_struct, u_int32_t *id, NDPI_PROTOCOL_BITMASK *detection_bitmask)
 {
   ndpi_set_bitmask_protocol_detection("Skype", ndpi_struct, detection_bitmask, *id,
 				      NDPI_PROTOCOL_SKYPE,
